@@ -17,18 +17,25 @@ using std::ios;
 
 namespace fmesh {
 
-// enum Datatype {Datatype_dense=0,
-//                 Datatype_sparse=1};
-//  /*! int/double */
-//  enum Valuetype {Valuetype_int=0,
-//                  Valuetype_double=1};
-//  /*! general/symmentric/diagonal */
-//  enum Matrixtype {Matrixtype_general=0,
-//                   Matrixtype_symmetric=1,
-//                   Matrixtype_diagonal=2};
-//  /*! rowmajor/colmajor */
-//  enum Storagetype {Storagetype_rowmajor=0,
-//                    Storagetype_colmajor=1};
+// /*! dense/sparse/map */
+// enum class IODatatype : int {
+//   Invalid = -1,
+//     Dense = 0,
+//     Sparse = 1,
+//     Collection = 2
+// };
+// /*! int/double */
+// enum class IOValuetype : int {
+//   Invalid = -1,
+//     Int = 0,
+//     Double = 1
+// };
+// /*! rowmajor/colmajor */
+// enum class IOStoragetype : int {
+//   Invalid = -1,
+//     Rowmajor = 0,
+//     Colmajor = 1
+// };
 
 
 // No need for IOHeader and IOHelper classes when using Rcpp
@@ -40,14 +47,14 @@ IOHeader::IOHeader() { def(); }
 IOHeader &IOHeader::def(const int &ref) {
   (void)(ref);
   def();
-  valuetype = IOValuetype_int;
+  valuetype = IOValuetype::Int;
   return *this;
 }
 
 IOHeader &IOHeader::def(const double &ref) {
   (void)(ref);
   def();
-  valuetype = IOValuetype_double;
+  valuetype = IOValuetype::Double;
   return *this;
 }
 
@@ -56,19 +63,19 @@ IOHeader &IOHeader::def() {
   elems = 0;
   rows = 0;
   cols = 0;
-  datatype = -1;
-  valuetype = -1;
-  matrixtype = -1;
-  storagetype = IOStoragetype_rowmajor;
+  datatype = IODatatype::Invalid;
+  valuetype = IOValuetype::Invalid;
+  matrixtype = IOMatrixtype::Invalid;
+  storagetype = IOStoragetype::Rowmajor;
   return *this;
 }
 
 IOHeader &IOHeader::collection(const MatrixC &C) {
-  datatype = IODatatype_collection;
+  datatype = IODatatype::Collection;
   elems = C.output_size();
   rows = -1;
   cols = -1;
-  storagetype = -1;
+  storagetype = IOStoragetype::Invalid;
   return *this;
 }
 
@@ -89,7 +96,7 @@ std::istream &operator>>(std::istream &input, IOHeader &h) {
   int file_header_length;
   input >> file_header_length;
   if (file_header_length < header_length) {
-    h.dense(Matrix<int>(0), IOMatrixtype_general);
+    h.dense(Matrix<int>(0), IOMatrixtype::General);
     int *ioheader_p = (int *)&h;
     for (int i = 0; i < file_header_length; i++)
       input >> ioheader_p[i];
@@ -113,14 +120,13 @@ IOHelperC &IOHelperC::OL(std::ostream &output) {
   if ((h.elems == 0) || (!cM_)) {
     return *this;
   }
-  for (MatrixC::outputT::const_iterator outi = cM_->output_.begin();
-       outi != cM_->output_.end(); ++outi) {
+  for (const auto& outi : cM_->output_) {
     if (bin_) {
-      int string_size = (*outi).length() + 1;
+      int string_size = outi.length() + 1;
       output.write((char *)&string_size, sizeof(string_size));
-      output.write((char *)(*outi).c_str(), sizeof(char) * string_size);
+      output.write((char *)outi.c_str(), sizeof(char) * string_size);
     } else {
-      output << *outi << std::endl;
+      output << outi << std::endl;
     }
   }
   return *this;
@@ -155,11 +161,10 @@ IOHelperC &IOHelperC::OD(std::ostream &output) {
   if ((h.elems == 0) || (!cM_)) {
     return *this;
   }
-  for (MatrixC::outputT::const_iterator outi = cM_->output_.begin();
-       outi != cM_->output_.end(); ++outi) {
-    const MCC &mcc = *(cM_->coll_.find(*outi)->second);
-    if (mcc.info.datatype == IODatatype_dense)
-      if (mcc.info.valuetype == IOValuetype_int) {
+  for (const auto& outi : cM_->output_) {
+    const MCC &mcc = *(cM_->coll_.find(outi)->second);
+    if (mcc.info.datatype == IODatatype::Dense)
+      if (mcc.info.valuetype == IOValuetype::Int) {
         IOHelperM<int> ioh;
         ioh.cD(&(mcc.DI())).matrixtype(mcc.info.matrixtype);
         ioh.binary(bin_).OH(output).OD(output);
@@ -168,7 +173,7 @@ IOHelperC &IOHelperC::OD(std::ostream &output) {
         ioh.cD(&mcc.DD()).matrixtype(mcc.info.matrixtype);
         ioh.binary(bin_).OH(output).OD(output);
       }
-    else if (mcc.info.valuetype == IOValuetype_int) {
+    else if (mcc.info.valuetype == IOValuetype::Int) {
       IOHelperSM<int> ioh;
       ioh.cD(&mcc.SI()).matrixtype(mcc.info.matrixtype);
       ioh.binary(bin_).OH(output).OD(output);
@@ -187,28 +192,28 @@ IOHelperC &IOHelperC::ID(std::istream &input) {
   if ((!M_) || (h.elems == 0)) {
     return *this;
   }
-  for (listT::iterator listi = list_.begin(); listi != list_.end(); ++listi) {
+  for (const auto& listi : list_) {
 
     IOHelper<int> ioh_;
     ioh_.binary(bin_).IH(input);
 
-    if (ioh_.h_.datatype == IODatatype_dense)
-      if (ioh_.h_.valuetype == IOValuetype_int) {
+    if (ioh_.h_.datatype == IODatatype::Dense)
+      if (ioh_.h_.valuetype == IOValuetype::Int) {
         IOHelperM<int> ioh;
-        ioh.D(&(M_->DI(*listi)));
+        ioh.D(&(M_->DI(listi)));
         ioh.binary(bin_).IH(ioh_.h_).ID(input);
       } else {
         IOHelperM<double> ioh;
-        ioh.D(&(M_->DD(*listi)));
+        ioh.D(&(M_->DD(listi)));
         ioh.binary(bin_).IH(ioh_.h_).ID(input);
       }
-    else if (ioh_.h_.valuetype == IOValuetype_int) {
+    else if (ioh_.h_.valuetype == IOValuetype::Int) {
       IOHelperSM<int> ioh;
-      ioh.D(&(M_->SI(*listi)));
+      ioh.D(&(M_->SI(listi)));
       ioh.binary(bin_).IH(ioh_.h_).ID(input);
     } else {
       IOHelperSM<double> ioh;
-      ioh.D(&(M_->SD(*listi)));
+      ioh.D(&(M_->SD(listi)));
       ioh.binary(bin_).IH(ioh_.h_).ID(input);
     }
   }
@@ -223,7 +228,7 @@ template <>
 Matrix<int> &MatrixC::attach(std::string name, Matrix<int> *M,
                              bool transfer_ownership, IOMatrixtype matrixt) {
   free(name);
-  coll_.insert(collPairT(name, new MCC(IODatatype_dense, IOValuetype_int,
+  coll_.insert(collPairT(name, new MCC(IODatatype::Dense, IOValuetype::Int,
                                        matrixt, M, transfer_ownership)));
   activate(name);
   return coll_[name]->DI();
@@ -233,7 +238,7 @@ template <>
 Matrix<double> &MatrixC::attach(std::string name, Matrix<double> *M,
                                 bool transfer_ownership, IOMatrixtype matrixt) {
   free(name);
-  coll_.insert(collPairT(name, new MCC(IODatatype_dense, IOValuetype_double,
+  coll_.insert(collPairT(name, new MCC(IODatatype::Dense, IOValuetype::Double,
                                        matrixt, M, transfer_ownership)));
   activate(name);
   return coll_[name]->DD();
@@ -244,7 +249,7 @@ SparseMatrix<int> &MatrixC::attach(std::string name, SparseMatrix<int> *M,
                                    bool transfer_ownership,
                                    IOMatrixtype matrixt) {
   free(name);
-  coll_.insert(collPairT(name, new MCC(IODatatype_sparse, IOValuetype_int,
+  coll_.insert(collPairT(name, new MCC(IODatatype::Sparse, IOValuetype::Int,
                                        matrixt, M, transfer_ownership)));
   activate(name);
   return coll_[name]->SI();
@@ -255,7 +260,7 @@ SparseMatrix<double> &MatrixC::attach(std::string name, SparseMatrix<double> *M,
                                       bool transfer_ownership,
                                       IOMatrixtype matrixt) {
   free(name);
-  coll_.insert(collPairT(name, new MCC(IODatatype_sparse, IOValuetype_double,
+  coll_.insert(collPairT(name, new MCC(IODatatype::Sparse, IOValuetype::Double,
                                        matrixt, M, transfer_ownership)));
   activate(name);
   return coll_[name]->SD();
@@ -271,8 +276,8 @@ bool MatrixC::activate(std::string name) {
 }
 
 void MatrixC::activate() {
-  for (collT::iterator colli = coll_.begin(); colli != coll_.end(); ++colli) {
-    colli->second->info.active = true;
+  for (const auto& colli : coll_) {
+    colli.second->info.active = true;
   }
 }
 
@@ -299,9 +304,8 @@ void MatrixC::load_file(std::string filename, bool only_list) {
   }
 
   /* Populate source_ */
-  for (IOHelperC::listT::const_iterator listi = ioh.list_.begin();
-       listi != ioh.list_.end(); ++listi) {
-    source_[(*listi)] = filename;
+  for (const auto& listi : ioh.list_) {
+    source_[listi] = filename;
   }
 }
 
@@ -344,8 +348,8 @@ MCCInfo MatrixC::load(std::string name) {
   if (!I.is_open()) {
     return info(name);
   }
-  if (ioh_.h_.datatype == IODatatype_dense)
-    if (ioh_.h_.valuetype == IOValuetype_int) {
+  if (ioh_.h_.datatype == IODatatype::Dense)
+    if (ioh_.h_.valuetype == IOValuetype::Int) {
       IOHelperM<int> ioh;
       ioh.D(&DI(name));
       ioh.binary(bin_in_).IH(I);
@@ -355,7 +359,7 @@ MCCInfo MatrixC::load(std::string name) {
       ioh.D(&DD(name));
       ioh.binary(bin_in_).IH(I).ID(I);
     }
-  else if (ioh_.h_.valuetype == IOValuetype_int) {
+  else if (ioh_.h_.valuetype == IOValuetype::Int) {
     IOHelperSM<int> ioh;
     ioh.D(&SI(name));
     ioh.binary(bin_in_).IH(I).ID(I);
@@ -393,15 +397,15 @@ MatrixC &MatrixC::dont_output(std::string name) {
 MatrixC &MatrixC::output(std::string name) {
   if (name == "-") {
     output_all_ = true;
-    for (collT::iterator colli = coll_.begin(); colli != coll_.end(); ++colli) {
-      if (colli->second->info.active)
-        output_.insert(colli->first);
+    for (auto const & colli : coll_) {
+      if (colli.second->info.active)
+        output_.insert(colli.first);
     }
   } else if (name == "--") {
     output_all_ = true;
-    for (collT::iterator colli = coll_.begin(); colli != coll_.end(); ++colli) {
-      if (activate(colli->first))
-        output_.insert(colli->first);
+    for (auto const & colli : coll_) {
+      if (activate(colli.first))
+        output_.insert(colli.first);
     }
   } else {
     if (info(name).loaded) {
@@ -470,18 +474,17 @@ void MatrixC::input_raw(std::string name, std::string specification,
 void MatrixC::save() {
   /* Write the matrix collection to output */
   if (output_prefix_ != "-") {
-    for (outputT::const_iterator outi = output_.begin(); outi != output_.end();
-         ++outi) {
-      MCC &mcc = *(coll_.find(*outi)->second);
-      if (mcc.info.datatype == IODatatype_dense)
-        if (mcc.info.valuetype == IOValuetype_int)
-          save_M((output_prefix_ + (*outi)), mcc.DI(), mcc.info, bin_out_);
+    for (auto const & outi : output_) {
+      MCC &mcc = *(coll_.find(outi)->second);
+      if (mcc.info.datatype == IODatatype::Dense)
+        if (mcc.info.valuetype == IOValuetype::Int)
+          save_M((output_prefix_ + outi), mcc.DI(), mcc.info, bin_out_);
         else
-          save_M((output_prefix_ + (*outi)), mcc.DD(), mcc.info, bin_out_);
-      else if (mcc.info.valuetype == IOValuetype_int)
-        save_SM((output_prefix_ + (*outi)), mcc.SI(), mcc.info, bin_out_);
+          save_M((output_prefix_ + outi), mcc.DD(), mcc.info, bin_out_);
+      else if (mcc.info.valuetype == IOValuetype::Int)
+        save_SM((output_prefix_ + outi), mcc.SI(), mcc.info, bin_out_);
       else
-        save_SM((output_prefix_ + (*outi)), mcc.SD(), mcc.info, bin_out_);
+        save_SM((output_prefix_ + outi), mcc.SD(), mcc.info, bin_out_);
     }
   }
   if (output_file_ != "") {
@@ -508,8 +511,8 @@ void MatrixC::save() {
 Matrix<int> &MatrixC::DI(std::string name) {
   collT::iterator colli;
   if (((colli = coll_.find(name)) != coll_.end()) &&
-      (colli->second->info.datatype == IODatatype_dense) &&
-      (colli->second->info.valuetype == IOValuetype_int) &&
+      (colli->second->info.datatype == IODatatype::Dense) &&
+      (colli->second->info.valuetype == IOValuetype::Int) &&
       (colli->second->info.active)) {
     return colli->second->DI();
   }
@@ -519,8 +522,8 @@ Matrix<int> &MatrixC::DI(std::string name) {
 Matrix<double> &MatrixC::DD(std::string name) {
   collT::iterator colli;
   if (((colli = coll_.find(name)) != coll_.end()) &&
-      (colli->second->info.datatype == IODatatype_dense) &&
-      (colli->second->info.valuetype == IOValuetype_double) &&
+      (colli->second->info.datatype == IODatatype::Dense) &&
+      (colli->second->info.valuetype == IOValuetype::Double) &&
       (colli->second->info.active)) {
     return colli->second->DD();
   }
@@ -530,8 +533,8 @@ Matrix<double> &MatrixC::DD(std::string name) {
 SparseMatrix<int> &MatrixC::SI(std::string name) {
   collT::iterator colli;
   if (((colli = coll_.find(name)) != coll_.end()) &&
-      (colli->second->info.datatype == IODatatype_sparse) &&
-      (colli->second->info.valuetype == IOValuetype_int) &&
+      (colli->second->info.datatype == IODatatype::Sparse) &&
+      (colli->second->info.valuetype == IOValuetype::Int) &&
       (colli->second->info.active)) {
     return colli->second->SI();
   }
@@ -541,8 +544,8 @@ SparseMatrix<int> &MatrixC::SI(std::string name) {
 SparseMatrix<double> &MatrixC::SD(std::string name) {
   collT::iterator colli;
   if (((colli = coll_.find(name)) != coll_.end()) &&
-      (colli->second->info.datatype == IODatatype_sparse) &&
-      (colli->second->info.valuetype == IOValuetype_double) &&
+      (colli->second->info.datatype == IODatatype::Sparse) &&
+      (colli->second->info.valuetype == IOValuetype::Double) &&
       (colli->second->info.active)) {
     return colli->second->SD();
   }
@@ -568,19 +571,17 @@ MCCInfo MatrixC::info(std::string name) const {
 SEXP MatrixC::Rcpp_wrap() const {
   /* Convert the matrix collection to a list of R objects */
   Rcpp::List res;
-  for (outputT::const_iterator outi = output_.begin();
-       outi != output_.end();
-       ++outi) {
-    const MCC &mcc = *(coll_.find(*outi)->second);
-    if (mcc.info.datatype == IODatatype_dense) {
-      if (mcc.info.valuetype == IOValuetype_int)
-        res[(*outi)] = Rcpp::wrap(mcc.DI());
+  for (auto const & outi : output_) {
+    const MCC &mcc = *(coll_.find(outi)->second);
+    if (mcc.info.datatype == IODatatype::Dense) {
+      if (mcc.info.valuetype == IOValuetype::Int)
+        res[outi] = Rcpp::wrap(mcc.DI());
       else
-        res[(*outi)] = Rcpp::wrap(mcc.DD());
-    } else if (mcc.info.valuetype == IOValuetype_int) {
-      res[(*outi)] = Rcpp::wrap(mcc.SI());
+        res[outi] = Rcpp::wrap(mcc.DD());
+    } else if (mcc.info.valuetype == IOValuetype::Int) {
+      res[outi] = Rcpp::wrap(mcc.SI());
     } else
-      res[(*outi)] = Rcpp::wrap(mcc.SD());
+      res[outi] = Rcpp::wrap(mcc.SD());
   }
   return res;
 }
@@ -591,29 +592,29 @@ void MatrixC::attach(std::string name, SEXP from) {
     (*this).attach(name, new Matrix<double>(
         Rcpp::as<Rcpp::NumericMatrix>(
           from)), true,
-          IOMatrixtype_general);
+          IOMatrixtype::General);
   } else if (Rcpp::is<Rcpp::IntegerMatrix>(from)) {
     (*this).attach(name, new Matrix<int>(
         Rcpp::as<Rcpp::IntegerMatrix>(
           from)), true,
-          IOMatrixtype_general);
+          IOMatrixtype::General);
   } else if (Rcpp::is<Rcpp::CharacterMatrix>(from)) {
   } else if (Rcpp::is<Rcpp::NumericVector>(from)) {
     (*this).attach(name, new Matrix1<double>(
         Rcpp::as<Rcpp::NumericVector>(
           from)), true,
-          IOMatrixtype_general);
+          IOMatrixtype::General);
   } else if (Rcpp::is<Rcpp::IntegerVector>(from)) {
     (*this).attach(name, new Matrix1<int>(
         Rcpp::as<Rcpp::IntegerVector>(
           from)), true,
-          IOMatrixtype_general);
+          IOMatrixtype::General);
   } else if (Rcpp::is<Rcpp::CharacterVector>(from)) {
   } else {
     (*this).attach(name,
      new SparseMatrix<double>(from),
      true,
-     IOMatrixtype_general);
+     IOMatrixtype::General);
   }
 }
 
@@ -623,10 +624,8 @@ void MatrixC::attach(std::string name, SEXP from) {
 void MatrixC::attach(SEXP from) {
     Rcpp::List from_list = Rcpp::as<Rcpp::List>(from);
   Rcpp::CharacterVector from_names = from_list.names();
-  for (auto elem = from_names.begin();
-       elem != from_names.end();
-       elem++) {
-    std::string the_name = Rcpp::as<std::string>(*elem);
+  for (auto const & elem : from_names) {
+    std::string the_name = Rcpp::as<std::string>(elem);
     (*this).attach(the_name, from_list[the_name]);
   }
 }
